---
stage: Create
group: Code Review
info: To determine the technical writer assigned to the Stage/Group associated with this page, see https://about.gitlab.com/handbook/product/ux/technical-writing/#assignments
---

<!--
This documentation is auto generated by a script.
Please do not edit this file directly. Run `make gen-docs` instead.
-->

# `glab release create`

Create a new GitLab release, or update an existing one.

## Synopsis

Create a new release, or update an existing GitLab release, for a repository. Requires the Developer role or higher.

An existing release is updated with the new information you provide.

To create a release from an annotated Git tag, first create one locally with
Git, push the tag to GitLab, then run this command.

If the Git tag you specify doesn't exist, the release is created
from the latest state of the default branch, and tagged with the tag name you specify.

To override this behavior, use `--ref`. The `ref` can be a commit SHA, another tag name, or a branch name.

To fetch the new tag locally after the release, run `git fetch --tags origin`.

```plaintext
glab release create <tag> [<files>...] [flags]
```

## Examples

```plaintext
# Interactively create a release
$ glab release create v1.0.1

# Non-interactively create a release by specifying a note
$ glab release create v1.0.1 --notes "bugfix release"

# Use release notes from a file
$ glab release create v1.0.1 -F changelog.md

# Upload a release asset with a display name (type will default to 'other')
$ glab release create v1.0.1 '/path/to/asset.zip#My display label'

# Upload a release asset with a display name and type
$ glab release create v1.0.1 '/path/to/asset.png#My display label#image'

# Upload all assets in a specified folder (types will default to 'other')
$ glab release create v1.0.1 ./dist/*

# Upload all tarballs in a specified folder (types will default to 'other')
$ glab release create v1.0.1 ./dist/*.tar.gz

# Create a release with assets specified as JSON object
$ glab release create v1.0.1 --assets-links='
  [
    {
      "name": "Asset1",
      "url":"https://<domain>/some/location/1",
      "link_type": "other",
      "direct_asset_path": "path/to/file"
    }
  ]'

```

## Options

```plaintext
  -a, --assets-links --assets-links='[{"name": "Asset1", "url":"https://<domain>/some/location/1", "link_type": "other", "direct_asset_path": "path/to/file"}]'.   'JSON' string representation of assets links, like --assets-links='[{"name": "Asset1", "url":"https://<domain>/some/location/1", "link_type": "other", "direct_asset_path": "path/to/file"}]'.
  -m, --milestone strings                                                                                                                                          The title of each milestone the release is associated with.
  -n, --name string                                                                                                                                                The release name or title.
  -N, --notes string                                                                                                                                               The release notes or description. You can use Markdown.
  -F, --notes-file string                                                                                                                                          Read release notes 'file'. Specify '-' as the value to read from stdin.
  -r, --ref string                                                                                                                                                 If the specified tag doesn't exist, the release is created from ref and tagged with the specified tag name. It can be a commit SHA, another tag name, or a branch name.
  -D, --released-at string                                                                                                                                         The 'date' when the release was ready. Defaults to the current datetime. Expects ISO 8601 format (2019-03-15T08:00:00Z).
```

## Options inherited from parent commands

```plaintext
      --help              Show help for this command.
  -R, --repo OWNER/REPO   Select another repository. Can use either OWNER/REPO or `GROUP/NAMESPACE/REPO` format. Also accepts full URL or Git URL.
```
